

	LIST	p=16F84A	;tell assembler what chip we are using
	include "P16F84A.inc"		;include the defaults for the chip
	ERRORLEVEL	0,	-302	;suppress bank selection messages
	__config _CP_OFF & _PWRTE_OFF & _WDT_OFF & _XT_OSC

#DEFINE BANK0 bcf STATUS,5
#DEFINE BANK1 bsf STATUS,5


		cblock	0x20			;start of general purpose registers
			count			;used in looping routines
			count1			;used in delay routine
			counta			;used in delay routine
			countb			;used in delay routine
			tmp1			;temporary storage
			tmp2
			templcd			;temp store for 4 bit mode
			templcd2
			W_SAVE
			STAT_SV
			PCH_SV
			FSR_SV 
		endc

LCD_PORT	Equ	PORTA
LCD_TRIS	Equ	TRISA
;PORT B Control bit
LCD_RS		Equ	0x01			;RS line-> RB1
LCD_RW		Equ	0x02			;RW line-> RB2
LCD_E		Equ	0x03			;E  line-> RB3

		org	0x00
		goto	Start
		
		org     0x04

		movwf	W_SAVE          	; W_SAVE(bank unknown!) = W
         	swapf   STATUS,	W
         	clrf    STATUS          	; force bank 0 for remainder of handler
         	movwf   STAT_SV         	; STAT_SV = swap_nibbles( STATUS )
                                 		; STATUS = 0
         	movf    PCLATH,	W
         	movwf   PCH_SV          	; PCH_SV = PCLATH
         	clrf    PCLATH          	; PCLATH = 0
         	movf    FSR, W
         	movwf   FSR_SV          	; FSR_SV = FSR
         								; 12 cycles from interrupt to here!


ISR			btfss 	PORTB,7				;Determining which button cause the interupt
			call	LCD_Clr
			btfss 	PORTB,6
			call	Interupt1
			btfss 	PORTB,5
			call	Interupt2
			btfss 	PORTB,4
			call	Interupt2
		

ENDINT   	bcf		INTCON,RBIF		;Clear the flag to enable more interupts
			movf    FSR_SV,W
      		movwf   FSR             	; FSR = FSR_SV
         	movf    PCH_SV,	W
         	movwf   PCLATH          	; PCLATH = PCH_SV
         	swapf   STAT_SV,W
         	movwf   STATUS          	; STATUS = swap_nibbles( STAT_SV )
         	swapf   W_SAVE,	F
         	swapf   W_SAVE,	W     		; W = swap(swap( W_SAVE )) (no change Z bit)

         	retfie                  	; RETURN!

			org 0xff

Start		

Initialise	clrf	count
		clrf	PORTA
		clrf	PORTB
;Set Up The Interrupt Registers
		bsf     INTCON,GIE      	;GIE  Global interrupt enable (1=enable)
		bsf     INTCON,RBIE		;RBIE _ RB port change Interrupt Enable 
 		bcf     INTCON,RBIF     	;RBIF _ Clear FLag Bit

SetPorts	BANK1				;select bank 1
		movlw	0x00			;make all pins outputs
		movwf	LCD_TRIS
		movlw	b'11110000'		;set RB4-7 as input to use in Interrupt
		movwf	TRISB
		BANK0				;select bank 0
 		movlw	0xf0			;set RB4-7 to high		
		movwf	PORTB
		call	LCD_Init		;Initialise LCD
Loop		goto	Loop


Interupt1
		clrf	count			;set counter register to zero
Message1	movf	count, w		;move counter value in W
		call	Text3			;get a character from the text table
		xorlw	0x00			;is it a zero?
		btfsc	STATUS, Z		
		goto	NextMessage1
		call	PutChar
		call	Delay100
		incf	count, f
		goto	Message1

NextMessage1	movlw   0x03
		call	LCD_Line2W		;move to 2nd row, 4th column

		clrf	count			;set counter register to zero
Message1_2	movf	count, w		;put counter value in W
		call	Text4			;get a character from the text table
		xorlw	0x00			;is it a zero?
		btfsc	STATUS, Z
		goto	EndMessage1
		call	PutChar
		call	Delay100
		incf	count, f
		goto	Message1_2

EndMessage1	goto	ENDINT			;endless loop


Interupt2	call	LCD_Line1
		clrf	count			;set counter register to zero
Message2	movf	count, w		;move counter value in W
		call	Text1			;get a character from the text table
		xorlw	0x00			;is it a zero?
		btfsc	STATUS, Z		
		goto	NextMessage2
		call	PutChar
		call	Delay100
		incf	count, f
		goto	Message2

NextMessage2	movlw	0x03
		call	LCD_Line2W		;move to 2nd row, 4th column

		clrf	count			;set counter register to zero
Message2_2	movf	count, w		;put counter value in W
		call	Text2			;get a character from the text table
		xorlw	0x00			;is it a zero?
		btfsc	STATUS, Z
		goto	EndMessage2
		call	PutChar
		call	Delay100
		incf	count, f
		goto	Message2_2

EndMessage2	goto	ENDINT			;endless loop



;Subroutines and text tables

;LCD routines

;Initialise LCD
LCD_Init	call 	LCD_Busy		;Wait for LCD to settle

		movlw	0x20			;Set 4 bit mode
		call	PutCmd

		movlw	0x28			;Set display shift
		call	PutCmd

		movlw	0x06			;Set display character mode
		call	PutCmd

		movlw	0x0c			;Set display on/off and cursor command
		call	PutCmd			;Set cursor off

		call	LCD_Clr			;clear display


		retlw	0x00

; command set routine
PutCmd		movwf	templcd
		swapf	templcd,w		;send upper nibble
		andlw	0x0f			;clear upper 4 bits of W
		movwf	LCD_PORT
		bcf	PORTB, LCD_RS		;RS line to 0->Instruction input mode
		call	Pulse_e			;Pulse the E line high

		movf	templcd,w		;send lower nibble
		andlw	0x0f			;clear upper 4 bits of W
		movwf	LCD_PORT
		bcf	PORTB, LCD_RS		;RS line to 0->Instruction input mode
		call	Pulse_e			;Pulse the E line high
		call 	LCD_Busy
		retlw	0x00

PutChar		movwf	templcd
		swapf	templcd,w 		;send upper nibble
		andlw	0x0f			;clear upper 4 bits of W
		movwf	LCD_PORT
		bsf	PORTB, LCD_RS		;RS line to 1->Data input mode
		call	Pulse_e			;Pulse the E line high

		movf	templcd,w		;send lower nibble
		andlw	0x0f			;clear upper 4 bits of W
		movwf	LCD_PORT
		bsf	PORTB, LCD_RS		;RS line to 1->Data input mode
		call	Pulse_e			;Pulse the E line high
		call 	LCD_Busy
		retlw	0x00

LCD_Line1	movlw	0x80			;move to 1st row, first column
		call	PutCmd
		retlw	0x00

LCD_Line2	movlw	0xc0			;move to 2nd row, first column
		call	PutCmd
		retlw	0x00

LCD_Line1W	addlw	0x80			;move to 1st row, column W
		call	PutCmd
		retlw	0x00

LCD_Line2W	addlw	0xc0			;move to 2nd row, column W
		call	PutCmd
		retlw	0x00


LCD_Clr		movlw	0x01			;Clear display
		call	PutCmd
		retlw	0x00



Delay255	movlw	0xff			;delay 255 mS
		goto	d0
Delay100	movlw	d'100'			;delay 100mS
		goto	d0
Delay50		movlw	d'50'			;delay 50mS
		goto	d0
Delay20		movlw	d'20'			;delay 20mS
		goto	d0
Delay5		movlw	0x05			;delay 5.000 ms (4 MHz clock)
d0		movwf	count1
d1		movlw	0xC7			;delay 1mS
		movwf	counta
		movlw	0x01
		movwf	countb
Delay_0
		decfsz	counta, f
		goto	$+2
		decfsz	countb, f
		goto	Delay_0

		decfsz	count1	,f
		goto	d1
		retlw	0x00

Pulse_e		bsf	PORTB, LCD_E
		nop
		bcf	PORTB, LCD_E
		retlw	0x00

LCD_Busy
		BANK1				;set bank 1
		movlw	0x0f			;set Port for input
		movwf	LCD_TRIS
		BANK0				;set bank 0
		bcf	PORTB, LCD_RS		;set LCD for command mode
		bsf	PORTB, LCD_RW		;setup to read busy flag
		bsf	PORTB, LCD_E
		swapf	LCD_PORT, w		;read upper nibble (busy flag)
		bcf	PORTB, LCD_E		
		movwf	templcd2 
		bsf	PORTB, LCD_E		;dummy read of lower nibble
		bcf	PORTB, LCD_E
		btfsc	templcd2, 7		;check busy flag, high = busy
		goto	LCD_Busy		;if busy check again
		bcf	PORTB, LCD_RW
		BANK1				;set bank 1
		movlw	0x00			;set Port for output
		movwf	LCD_TRIS
		BANK0				;set bank 0
		return

			org 0x050

Text1		addwf	PCL, f
		retlw	'H'
		retlw	'u'
		retlw	'o'
		retlw	'n'
		retlw	'g'
		retlw	' '
		retlw	'd'
		retlw	'a'
		retlw	'n'
		retlw   0x00

Text2		addwf   PCL, f
            	retlw   's'
            	retlw   'u'
            	retlw   ' '
            	retlw   'd'
            	retlw   'u'
            	retlw   'n'
            	retlw   'g'
            	retlw   ' '
		retlw   'P'
		retlw   'I'
		retlw   'C'
            	retlw   0x00

Text3		addwf	PCL, f
		retlw	'P'
		retlw	'I'
		retlw	'C'
		retlw	' '
		retlw	'T'
		retlw	'u'
		retlw	't'
		retlw	'o'
		retlw	'r'
		retlw	'i'
		retlw	'a'
		retlw	'l'
		retlw	0x00

Text4		addwf   PCL, f
            	retlw   'L'
            	retlw   'C'
            	retlw   'D'
            	retlw   ' '
            	retlw   'T'
            	retlw   'C'
            	retlw   '1'
            	retlw   '6'
		retlw   '0'
		retlw   '1'
		retlw   'A'
            	retlw   0x00



;end of LCD routines

		end
